#include "GetArguments.h"

// Initialises with the default values
argsList application_arguments = 
{
	.references_output_file_location = "References.txt",
	.replacement_token = "Send",
	.debug_token = "DEBUG",
	.temp_file_token = ".temp"
};

// Makes the argument cases a bit clearer and slightly easier to add more
#define AddArgumentCase(character, variable) case character: \
											{\
											variable = *(argv + arg_counter + 1);\
											break;\
											}

/**
 * @brief Goes through the arguments passed in on the command line to parse them and put them into the arguments list for further use
 * 
 * @param app_arguments Pointer to struct containing all the possible arguments, initialised with defaults
 * @param argc The number of argument passed in on the command line
 * @param argv List of pointers to the arguments passed in
 * 
 * @return Number of input file path arguments found
 */
uint8_t CheckArguments (argsList * app_arguments, int argc, char ** argv)
{
	/*
	 * Skip the first argument on the list, it's just the path.
	 * If you have more then 255 arguments then there's bigger problems
	 */
	uint8_t arg_counter = 1;
	app_arguments->args_list_input_files_indexs = (uint8_t *) malloc(sizeof(uint8_t) * argc);
	
	uint8_t input_file_list_counter = 0;


	while (arg_counter < argc)
	{
		char * next_arg = *(argv + arg_counter);
		if (*next_arg == '-')
		{
			// Each argument is prepended with a marker, i.e. -
			char arg_type  = *(next_arg + 1);
			switch (arg_type)
			{
			AddArgumentCase('O', app_arguments->references_output_file_location);
			AddArgumentCase('D', app_arguments->debug_token);
			AddArgumentCase('R', app_arguments->replacement_token);
			AddArgumentCase('T', app_arguments->temp_file_token);
			default:
				printf("%s is not a valid argument.\n", next_arg);
				break;
			}

			// Next arg is the argument for the above case and so can be skipped 
			arg_counter++;
		} else
		{
			// If the arg isn't handled by the above then it is a file path, add it to the list
			*(app_arguments->args_list_input_files_indexs + input_file_list_counter) = arg_counter;
			input_file_list_counter++;
		}
		arg_counter++;
	}
	
	// Terminating NULL
	*(app_arguments->args_list_input_files_indexs + input_file_list_counter) = 0;

	return input_file_list_counter;
}

/**
 * @brief Prints all the input file paths
 * 
 * @param app_arguments Struct holding all the application arguments passed over the command line
 * @param argv List of string input arguments
 */
void PrintInputFilePaths(argsList * app_arguments, char ** argv)
{
	printf("Input file paths:\n");
	while(1)
	{
		static uint8_t i = 0;
		uint8_t argument_index = *(application_arguments.args_list_input_files_indexs + i);
		if (!argument_index)
		{
			break;
		}
		printf("%hhu : %s\n", i, *(argv + argument_index));
		i++;
	}
	return;
}