#include "GetArguments.h"

// Initialises with the default values
argsList application_arguments = 
{
	.ref_file_location = "References.txt",
	.com_port = "COM101",
	.baudrate = 115200,
	.test_string = "Test string"
};

// Makes the argument cases a bit clearer and slightly easier to add
#define AddArgumentCase(character, variable) case character: \
											{\
											variable = *(argv + arg_counter + 1);\
											break;\
											}

/**
 * @brief Goes through the arguments passed in on the command line to parse them and put them into the arguments list for further use
 * 
 * @param app_arguments Pointer to struct containing all the possible arguments, initialised with defaults
 * @param argc The number of argument passed in on the command line
 * @param argv List of pointers to the arguments passed in
 */
void CheckArguments (argsList * app_arguments, int argc, char ** argv)
{
	// Skip the first argument on the list, it's just the path
	uint8_t arg_counter = 1;

	while (arg_counter < argc)
	{
		char * next_arg = *(argv + arg_counter);
		if (*next_arg == '-')
		{
			// Each argument is prepended with a marker, i.e. -
			char arg_type  = *(next_arg + 1);
			switch (arg_type)
			{
			AddArgumentCase('R', app_arguments->ref_file_location);
			AddArgumentCase('C', app_arguments->com_port);
			AddArgumentCase('T', app_arguments->test_string);
			case 'B':
			{
				// Baudrate, needs a bit more processing then the other arguments
				// Try to get an integer from the string
				uint32_t baud_result = (uint32_t) atoi(*(argv + arg_counter + 1));
				if (baud_result)
				{
					app_arguments->baudrate = baud_result;
				}
				else
				{
					printf("Baudrate not valid, proceeding with default of %d\n", app_arguments->baudrate);
				}
				break;
			}
			default:
				printf("%s is not a valid argument.\n", next_arg);
				break;
			}
		}
		arg_counter++;
	}
	/*
	printf("Program parameters:\n\tReferences = %s\n\tCOM Port = %s\n\tBaudrate = %d\n\tTest string = %s\n", 
			app_arguments->ref_file_location,			
			app_arguments->com_port,			
			app_arguments->baudrate,			
			app_arguments->test_string
			);
	*/
	return;
}