#include "PrintDebugMessages.h"

static inline void GetHeaderInformation(FILE * references_file, referencesFileHeader * references_header);

/**
 * @brief Finds the related debug message and prints it out to the console
 * 
 * @param[in] messages_list Pointer to memory holding all messages 
 * @param[in] message_index Index of message to be printed out 
 * @param[in] references_header Pointer to struct containing all the metadata about the messages
 */
void PrintMessage(char * messages_list, uint8_t message_index, referencesFileHeader * references_header)
{
	if (message_index > references_header->number_debug_messages)
	{
		printf("Message %hhu doesn't exist.\n", message_index);
		return;
		// TO DO
		// return DEBUG_MESSAGE_DOESNT_EXIST;
	} else
	{
		// Need to use - 1 as message numbers are all one higher to avoid receiving a NULL over serial
		printf("%hu : %s", message_index, (messages_list + ((message_index - 1) * references_header->max_line_length)));
		return;
	}
}

/**
 * @brief Reads all the messages in the references file and puts them into a lump of memory. Also parses the file header.
 * 
 * @param[in] references_file  
 * @param[out] messages Address to write pointer for memory where all the messages are saved
 * @param[out] references_header Pointer to empty struct where all references metadata will be placed
 */
void CollectMessages(FILE * references_file, char ** messages, referencesFileHeader * references_header)
{
	GetHeaderInformation(references_file, references_header);

	// Allocate a lump of memory to hold all the messages together. Each message gets a piece of memory the size of the maximum message
	*messages =  (char *) malloc(sizeof(char) * references_header->max_line_length * references_header->number_debug_messages);
	
	// Dummy variable to fscanf() into.
	uint8_t message_number;
	for (uint8_t i = 0; i < references_header->number_debug_messages; i++)
	{
		fscanf(references_file, "%hhu : ", &message_number);
		// This increments the pointer for where in the large chunk of memory the next message should be placed
		// The buffer position is a bit redundant but makes it clearer.
		char * buffer_position = *messages + (i * references_header->max_line_length);
		fgets(buffer_position, references_header->max_line_length, references_file);
		// printf("%hhu message: |%s|", message_number, buffer_position);

	}

	return;
}

/**
 * @brief Reads the first line of metadata in the references file into a struct.
 * 
 * @param[in] references_file Poniter to file to read first line from
 * @param[out] references_header Pointer to struct to receive data 
 */
static inline void GetHeaderInformation(FILE * references_file, referencesFileHeader * references_header)
{
	fscanf(references_file, "Version : %hhu.%hhu, Debug messages : %hhu, Max line length : %hu\n",
				&references_header->version_major,
				&references_header->version_minor,
				&references_header->number_debug_messages,
				&references_header->max_line_length);
	return;
}