#include <stdio.h>
#include <stdint.h>
#include <Windows.h>
#include "SerialComms.h"
#include "GetArguments.h"
#include "PrintDebugMessages.h"
#include "../ReferencesFileInfo.h"

int main( int argc, char ** argv)
{
    CheckArguments(&application_arguments, argc, argv);

	FILE * references_file = fopen(application_arguments.ref_file_location, "r");
	if (!references_file)
	{
		printf("References file not found\n");
		return S_OK;
	}

	/// @brief Big lump of memory to hold all the messages at intervals of the max line size
	char * message_collection = NULL;
	/// @brief Struct with all the metadata about the references file
	referencesFileHeader references_information;

	CollectMessages(references_file, &message_collection, &references_information);
	printf("Preprocessor version is %hhu.%hhu\n", references_information.version_major, references_information.version_minor);

	HANDLE handle_serial_comm;  // Handle to the Serial port

	OpenSerialPort(&handle_serial_comm, application_arguments.com_port, application_arguments.baudrate);
	
	// SendOverSerial(&handle_serial_comm, application_arguments.test_string, TRUE);

	// Trigger COM event on characters in input buffer
	DWORD comm_event_mask = EV_RXCHAR;
	DWORD wait_comm_event_mask = 0;
	SetCommMask(handle_serial_comm, comm_event_mask);

	// The messages from the uC should be quite small, that's sort of the point of this whole thing
	const uint8_t message_buffer_increment_size = 8;
	
	// TODO: This is a fucking mess, it needs to be cleaned up and put into a function
	while(1)
	{
		uint32_t new_message_buffer_size = message_buffer_increment_size + 1;
		uint32_t new_message_buffer_start = 0;
		char * message_characters = (char *) malloc(sizeof(char) * new_message_buffer_size);

		if(WaitCommEvent(handle_serial_comm, &wait_comm_event_mask, NULL))
		{
			serialReturnCode status = SERIAL_RETURN_OK;

			while (1)
			{
				if(message_characters)
				{
					status = ReceiveOverSerial(&handle_serial_comm, (message_characters + new_message_buffer_start), message_buffer_increment_size);

					if (status != SERIAL_BUFFER_NOT_FULLY_READ)
					{
						// printf("Fully read, status is %d\n", status);
						break;
					} else
					{
						// printf("Not fully read, status is %d\n", status);
						new_message_buffer_size += sizeof(char) * message_buffer_increment_size;
						new_message_buffer_start += sizeof(char) * message_buffer_increment_size;
						message_characters = (char *) realloc(message_characters, new_message_buffer_size);
					}
				} else
				{
					printf("Memory error\n");
					return 1;
				}

			}
			// printf("Message = %s and status is %d\n", message_characters, status);

			if (status == SERIAL_READ_ERROR)
			{
				printf("Serial read error\n");
				return 1;
			}
			

			uint8_t i = 0;
			while (*(message_characters + i))
			{
				PrintMessage(message_collection, (uint8_t) (*(message_characters + i)) - 1, &references_information);
				i++;
			}
			free(message_characters);
		}
	}
    
	return 0;

}

